<!DOCTYPE html>
<html>

<head>
    <link type="text/css" rel="stylesheet" href='{{ swagger_ui_css }}'>
    <link rel="shortcut icon" href='{{ swagger_ui_favicon }}'>
    <title>{{title}}</title>
</head>

<body>
    <div id="swagger-ui">
    </div>
    <script src='{{ swagger_ui_js }}'></script>
    <!-- `SwaggerUIBundle` is now available on the page -->
    <script>
        const ui = SwaggerUIBundle({
            url: '{{ openapi_url }}',
            oauth2RedirectUrl: window.location.origin + '/docs/oauth2-redirect',
            dom_id: '#swagger-ui',
            presets: [
                SwaggerUIBundle.presets.apis,
                SwaggerUIBundle.SwaggerUIStandalonePreset
            ],
            layout: "BaseLayout",
            deepLinking: true,
            showExtensions: true,
            showCommonExtensions: true
        })
    </script>
</body>

</html>